<?php
namespace komaphp\core;
/**
 * [Core]Form：入力フォーム部品コントローラ
 * 入力フォーム部品HTMLの生成を行う
 */
class Form{
	/**
	 * input type=text
	 * @param string $name /フォーム名（画面内一意であること）
	 * @param string $value
	 * @param array $args
	 * @param boolean $disp
	 */
	public static function text(string $name=null,string $value="",array $args = [],bool $disp = false):string
	{
		if($disp){
			return self::textCommon("label",$name,$value,$args);
		}else{
			return self::textCommon("text",$name,$value,$args);
		}
	}
	/**
	 * input type=password
	 * @param string $name /フォーム名（画面内一意であること）
	 * @param string $value
	 * @param array $args
	 * @param boolean $disp
	 */
	public static function password(string $name=null,string $value="",array $args = [],$disp = false):string
	{
		if($disp){
			return self::textCommon("label",$name,"********");
		}else{
			//セキュリティ対策：パスワードは再度打ち直させる
			return self::textCommon("password",$name,null,$args);
		}
	}
	/**
	 * label
	 * @param string $name /フォーム名（画面内一意であること）
	 * @param string $value
	 * @param array $args
	 */
	public static function label(string $name=null,string $value="",array $args = [],$disp=false):string
	{
		if(is_null($value) || $value == ""){
			$value = "-";
		}

		return self::textCommon("label",$name,$value,$args);
	}
	/**
	 * hidden
	 * @param string $name /フォーム名（画面内一意であること）
	 * @param string $value
	 * @param array $args
	 */
	public static function hidden(string $name=null,string $value="",$disp=false):string
	{
		return self::textCommon("hidden",$name,$value,[]);
	}
	/**
	 * text関係共通
	 * @param string $type
	 * @param string $name
	 * @param string $value
	 * @param [] $args
	 * @return string
	 */
	private static function textCommon($type,$name="",$value="",$args=[]):string
	{

		$value = Html::hsp($value);

		$option = null;
		if(count($args) > 0){
			foreach( $args as $k => $v ){
				$option .= " {$k}=\"$v\" ";
			}
		}

		//nameがnullの場合はラベルのみ表示
		if(is_null($name) || $name == ""){
			return "<span>".$value."</span>\n";
		}

		if($type=="label"){
			$hd = self::hidden($name,$value);
			return <<< EOF
			<span id="{$name}disp">{$value}</span>{$hd}
EOF;
		}

		return "<input type=\"{$type}\" value=\"{$value}\" name=\"{$name}\" id=\"{$name}\" {$option}>\n";
	}

	/**
	 * input type=button
	 * @param string $name /フォーム名（画面内一意であること）
	 * @param string $value
	 * @param array $args
	 */
	public static function button(string $name=null,string $value="push",array $args = []):string
	{
		return self::buttonCommon("button",$name,$value,$args);
	}
	/**
	 * input type=submit
	 * @param string $name /フォーム名（画面内一意であること）
	 * @param string $value
	 * @param array $args
	 */
	public static function submit(string $name=null,string $value="submit",array $args = []):string
	{
		return self::buttonCommon("submit",$name,$value,$args);
	}
	/**
	 * ボタン関係共通
	 * @param string $type
	 * @param string $name
	 * @param string $value
	 * @param array $args
	 * @return string
	 */
	private static function buttonCommon($type,$name,$value,$args):string{

		$option = null;
		if(count($args) > 0){
			foreach( $args as $k => $v ){
				$option .= " {$k}=\"$v\" ";
			}
		}

		return "<button type=\"{$type}\" name=\"{$name}\" id=\"{$name}\" {$option}>{$value}</button>\n";
	}

	/**
	 * textarea
	 * @param string $name
	 * @param string $value
	 * @param string $args
	 * @param string $disp
	 * @return string
	 */
	public static function textarea(string $name=null,string $value="",array $args=[],bool $disp=false):string
	{
		$option = null;
		if(count($args) > 0){
			foreach( $args as $k => $v ){
				$option .= " {$k}=\"$v\" ";
			}
		}

		//nameがnullの場合はラベルのみ表示
		if((is_null($name) || $name == "") || $disp){
			$hd = self::hidden($name,$value);
			if(is_null($value) || $value == ""){
				$value = "-";
			}
			$va = Html::tAreaForDisp($value,true);
			return "<p>".$va."</p>\n".$hd;
		}

		$va = Html::hsp($value);

		return "<textarea name=\"{$name}\" id=\"{$name}\" {$option}>{$va}</textarea>
		";
	}

	/**
	 * select
	 * @param string $name /フォーム名（画面内一意であること）
	 * @param array $select
	 * @param string $default
	 * @param array $args
	 * @param boolean $disp
	 * @param boolean $disp_key
	 */
	public static function select($name=null,$select=[],$default=null,$args = [],$disp = false ,$disp_key = false):string
	{
		if($name == null || $name == null || count($select) == 0 ) return null;

		$default = Html::hsp($default);

		$option = null;
		if(count($args) > 0){
			foreach( $args as $k => $v ){
				$option .= " {$k}=\"$v\" ";
			}
		}
		if($disp){
			$hd = self::hidden($name,$default);
			if(!is_null($default) && Helper::ak($default,$select)){
				return "<p id=\"{$name}disp\">".$select[$default]."</p>\n".$hd;
			}else{
				return "<p id=\"{$name}dispx\">-</p>\n".$hd;
			}
		}else{
			$html = "<select name=\"{$name}\" id=\"{$name}\" {$option}>\n";
				$html.="<option value=\" \" >　</option>\n";
			foreach($select as $k=>$v){
				$s="";
				if(!is_null($default) && $default == $k){
					$s="selected";
				}
				$html.="<option value=\"{$k}\" {$s}>{$v}</option>\n";
			}
			$html .= "</select>\n";
		}
		return $html;
	}
	/**
	 * radio
	 * @param string $name /フォーム名（画面内一意であること）
	 * @param array $select
	 * @param string $default
	 * @param array $args
	 * @param boolean $disp
	 * @param boolean $disp_key
	 */
	public static function radio($name=null,$select=[],$default=null,$args = [],$disp = false,$disp_key = false):string
	{

		if($name == null || $name == null || count($select) == 0 ) return null;

		$default = Html::hsp($default);

		if($disp && $name == "blog" && $default == "1" ){
			return Html::weblink("Blog","ブログ");
		}

		$option = null;
		if(count($args) > 0){
			foreach( $args as $k => $v ){
				$option .= " {$k}=\"$v\" ";
			}
		}
		if($disp){
			$hd = self::hidden($name,$default);
			if(!is_null($default) && Helper::ak($default,$select)){
				$select[$default] = Html::hsp($select[$default]);
				return "<p id=\"{$name}disp\">".$select[$default]."</p>\n".$hd;
			}else{
				return "<p id=\"{$name}disp\">-</p>\n".$hd;
			}
		}else{
			$html = null;
			$cnt=0;
			foreach($select as $k=>$v){
				$s="";
				if(!is_null($default) && $default == $k){
					$s="checked";
				}
				if($disp_key){
					$v = $k.":".$v;
				}
				//phpコードは置換
				$v = Html::hsp($v);
				$html.="<input type=\"radio\" name=\"{$name}\" id=\"{$name}{$cnt}\" value=\"{$k}\" {$s} {$option}><label for=\"{$name}{$cnt}\">{$v}</label>&nbsp;\n";
				if($disp_key){
					$html .= "<br />";
				}

				$cnt++;
			}
		}
		return $html;
	}
	/**
	 * check
	 * @param string $name /フォーム名（画面内一意であること）
	 * @param array $select
	 * @param string $default
	 * @param array $args
	 * @param boolean $disp
	 * @param boolean $disp_key
	 */
	public static function check($name=null,$select=[],$default=null,$args = [],$disp = false,$disp_key = false):string
	{
		if($name == null || $name == null || count($select) == 0 ) return null;

		$option = null;
		if(count($args) > 0){
			foreach( $args as $k => $v ){
				$option .= " {$k}=\"$v\" ";
			}
		}
		if($disp){
			if(!is_null($default) && Helper::ak($default,$select)){
				$ss = null;
				foreach($default as $v){
					if($disp_key){
						$ss .= $v.":";
					}
					$ss .= $select[$v]."&nbsp;";
				}
				//phpコードは置換
				$ss = Html::hsp($ss);

				return "<p id=\"{$name}disp\">".$ss."</p>\n";
			}else{
				return "<p id=\"{$name}disp\">-</p>\n";
			}
		}else{
			$html = null;
			$cnt=0;
			foreach($select as $k=>$v){
				$s="";
				if(!is_null($default)){
					foreach($default as $dv){
						if($dv == $k){
							$s="checked";
						}
					}
				}
				if($disp_key){
					$v = $k.":".$v;
				}
				$v = Html::hsp($v);

				$html.="<input type=\"checkbox\" name=\"{$name}[]\" id=\"{$name}{$cnt}\" value=\"{$k}\" {$s} {$option}><label for=\"{$name}{$cnt}\">{$v}</label>&nbsp;\n";

				if($disp_key){
					$html .= "<br />";
				}
				$cnt++;
			}
		}
		return $html;
	}

	/**
	 * file
	 * @param string $name /フォーム名（画面内一意であること）
	 * @param string $value /画像名
	 * @param array $args /設定
	 * @param boolean $disp /表示モード
	 */
	public static function file($name=null,$value="",$args = [],$disp = false):string
	{

		$value = Html::hsp($value);

		$option = null;
		if(count($args) > 0){
			foreach( $args as $k => $v ){
				$option .= " {$k}=\"$v\" ";
			}
		}

		$root = Config::get("app","root");

		$img = null;
		$button = null;
		if($value != ""){
			//削除ボタン
			if(!$disp){

				$delbtn = self::button("imgDel{$name}","削除",["onclick"=>"imgdel('{$name}');"]);
				$button = <<< EOF
{$delbtn}
<span class="help">差し替えたい場合は左のボタンで削除してください</span>
EOF;
			}
			return <<< EOF

			<img src="{$root}Include/{$name}/{$value}" id="{$name}disp" style="float:left;" />{$button}
			<div class="spacer10"></div>
EOF;
		}

		if($disp){
			//$hd = self::hidden($name,$value);
			return <<< EOF
			$img
EOF;
		}

		$html = <<< EOF
		{$img}
		<input type="file" name="{$name}" id="{$name}" {$option}>
EOF;
		return $html;
	}

}