<?php
namespace komaphp\core;
/**
 * [Core]Html：Html処理クラス
 * Html生成に関する機能
 *
 * @link http://komaphp.k-akimoto-matsu.net/Reference#subtitle0 リファレンス@Core.Html
 * @author TakanoriMatsushige < http://k-akimoto-matsu.net/Inquiry >
 * @copyright k_akimoto_matsu. All Rights Reserved
 */
class Html{

	/**
	 * a リンク
	 * @param string $href リンク先
	 * @param string $linkName 文言
	 * @param array $args
	 * @param boolean $on true:サイト内 false:外部
	 * @return string
	 */
	public static function link($href="",$linkName="link",$args =[],$on=true){
		$option = null;
		if(count($args) > 0){
			foreach( $args as $k => $v ){
				$option .= " {$k}=\"$v\" ";
			}
		}
		if($on){
			return "<a href=\"".Config::get("app","root")."{$href}\" {$option}>{$linkName}</a>\n";
		}else{
			return "<a href=\"{$href}\" {$option}>{$linkName}</a>\n";
		}
	}

	/**
	 * textarea系のコメントをWEBに表示できる形式にして返す
	 * @param string $value 対象文字列
	 * @param string $f htmlエスケープをするかどうか
	 * @return mixed
	 */
	public static function tAreaForDisp($value="",$f=false){
		$comment = "";
		if($f){
			$comment = str_replace(" ","&nbsp;",self::hsp($value));
		}else{
			$comment = $value;
		}
		$comment = str_replace("\t","&nbsp;&nbsp;&nbsp;&nbsp;",$comment);
		$comment = str_replace(["\r\n","\n"],"<br />",$comment);
		return $comment;
	}

	/**
	 * htmlspecialchars
	 * @param string $value
	 * @return string
	 */
	public static function hsp($value=""){
		if(is_array($value)) return $value; //配列は処理せずそのまま返す
		return htmlentities($value, ENT_QUOTES,'utf-8');
	}

	/**
	 * ページャーHTMLを生成する
	 * @param string $listName ページ名
	 * @param number $page 現在のページ
	 * @param number $count データ個数
	 */
	static function pager(string $listName,int $page=1,int $count=1,int $pager_count = 0){

	    $pager_count = ($pager_count===0)? Config::get("app","pager"):$pager_count;

	    $pagerHtml=null;
	    $pagerHtmls = [];

	    $start = 1 + ($pager_count * ($page - 1));
	    $pageCnt = ceil($count / $pager_count);
	    $pagerHtml = "<ul class=\"pagination\">\n";

	    if($page > 1){
	        if(($page-1) > 1){
	            $pagerHtmls[] = "<li class=\"waves-effect\">".Html::link("{$listName}/1",'<i class="material-icons">first_page</i>',["title"=>"最初へ","alt"=>"最初へ"])."</li>";
	        }
	        $pagerHtmls[] = "<li class=\"waves-effect\">".Html::link("{$listName}/".($page-1),'<i class="material-icons">chevron_left</i>',["title"=>($page-1)."ページ目へ","alt"=>($page-1)."ページ目へ"])."</li>";
	    }
	    $gn = 0;
	    if($pageCnt > Config::get("app","pagersplit")){
	        $l = null;
	        for($i=1;$i<=$pageCnt;$i++){
	            if($i < ($page - ( Config::get("app","pagersplit") / 2)) || $i > ($page + ( Config::get("app","pagersplit") / 2))) {
	                if($gn == 0){
	                    $gn = $i;
	                }

	                $pagerHtmls[$gn+1] = "<li class=\"waves-effect\">...</li>";
	                continue;
	            }
	            $l = $i;

	            if($i == $page){
	                $pagerHtmls[] = "<li class=\"active purple\">".Html::link("{$listName}/{$i}",$i)."</li>";
	            }else{
	                $pagerHtmls[] = "<li class=\"waves-effect\">".Html::link("{$listName}/{$i}",$i)."</li>";
	            }
	        }
	        if($l < $pageCnt){
	            $pagerHtmls[$i+1] = "<li class=\"waves-effect\">...</li>";
	        }
	    }else{
	        for($i=1;$i<=$pageCnt;$i++){
	            if($i == $page){
	                $pagerHtmls[] = "<li class=\"active purple\">".Html::link("{$listName}/{$i}",$i)."</li>";
	            }else{
	                $pagerHtmls[] = "<li class=\"waves-effect\">".Html::link("{$listName}/{$i}",$i)."</li>";
	            }
	        }
	    }

	    if($pageCnt != $page){
	        $pagerHtmls[] = "<li class=\"waves-effect\">".Html::link("{$listName}/".($page+1),'<i class="material-icons">chevron_right</i>',["title"=>($page+1)."ページ目へ","alt"=>($page+1)."ページ目へ"])."</li>";
	        if(($page+1) < $pageCnt){
	            $pagerHtmls[] = "<li class=\"waves-effect\">".Html::link("{$listName}/".$pageCnt,'<i class="material-icons">last_page</i>',["title"=>"最後へ","alt"=>"最後へ"])."</li>";
	        }
	    }
	    //最後が何番目か算出
	    $end = $start-1+Config::get("app","pager");
	    $endp = $page * Config::get("app","pager");
	    if(($count-$endp) < 0){
	        $end = (($page-1) * $pager_count) + ($count-(($page-1) * $pager_count));
	    }

	    $pagerHtmls[] = "<li class=\"result\">&nbsp;全".number_format($count)."件中".number_format($start)."～".number_format($end)."番目を表示</li>";

	    $pagerHtml .= join("\n",$pagerHtmls)."\n";
	    $pagerHtml .= "</ul>\n";
	    $pagerHtml .= "<div class=\"clear\"></div>\n";

	    return $pagerHtml;
	}

}
