<?php
namespace komaphp\core;

use komaphp\base as base;
use komaphp\fwboar\controllers as controllers;

/**
 * [Core]Run：メインロジック
 */
class Run
{
    private $urls =[];
    private $template =null;
    private $args =null;
    private $route = "";
    private $route_setting = [];
    private $request = [];
    private $response = [];

    public function main()
    {
        try{
            Session::start();
            Session::destroy("DBCon");
            $this->urlana();
            $this->setView();

            $this->getRouteSetting();
            $this->executeController();
            $this->show();
        }catch(base\RunException $e){
            $this->displayErrorPage($e->getMessage());
        }
    }
    private function getRouteSetting()
    {
        $router = require_once Dir::boar_path().DS.'route.php'; //ルーティング
        //１つ目はコントローラー名
        if(count($this->urls) == 0){
            $route = "index";
        }else{
            $route = $this->urls[0];
        }
        if(!Helper::ak($route,$router)){
            $msg = "wrong access dont exists router:".$route;
            throw new base\RunException($msg);
        }
        $this->route = $route;
        $this->route_setting = $router[$route];
    }

    /**
     * エラーページを表示する
     * @param string $msg
     * @return string
     */
    private function displayErrorPage(string $msg = null)
    {
        $loader = new \Twig\Loader\FilesystemLoader(Dir::views_path());
        $twig = new \Twig\Environment($loader, [
            'cache' => Dir::templates_path().DS.'cache',
        ]);
        echo $twig->render('error/error.html', ['msg' => $msg]);
        die();
    }
    private function setView()
    {
        //Dirのメソッドを全て変数名でアサイン
        $dir_method = get_class_methods("komaphp\\core\\Dir");
        
        $this->agrs = [];
        foreach($dir_method as $dir){
            $this->agrs[$dir] = Dir::$dir();
        }

        //coreを全てクラス名でアサイン
        foreach(glob(Dir::core_path().DS.'*') as $file){
            $classname = str_replace(Dir::core_path().DS,'',str_replace('.php','',$file));
            $obj = __NAMESPACE__ ."\\".$classname;
            $this->agrs[$classname] = new $obj;
        }

    }
    private function executeController(){
        //コントローラ決定
        $conName = $this->route_setting["controller"];
        $cFName = $conName."Controller";

        if(!file_exists(Dir::controllers_path().DS.$cFName.'.php')){
            $msg = "wrong access：".$cFName;
            throw new base\RunException($msg);
        }

        //認証が必要なシステム構築のときに利用する
        //self::accessAuth();
        $controller = "komaphp\\fwboar\\controllers\\".str_replace("/","\\",$cFName);

        if(is_null($controller) || !class_exists($controller)){
            $msg = "not exist controller";
            throw new base\RunException($msg.':'.$controller);
        }
        $class = new $controller;
        if($_SERVER['REQUEST_METHOD'] === 'POST'){
            $me = 'post';
        }else{
            $me = 'get';
        }
        //ルーティングからURLパラメータを取得
        $this->request['route_url'] = $this->route;
        $this->request['router'] = $this->route_setting;
        $this->request['route'] = [];
        if(Helper::ak('set', $this->route_setting)){
            foreach($this->urls as $k=>$v){
                if($k == 0) continue; //１つ目はルータ名のため回避
                if(Helper::ak($k, $this->route_setting['set'])){
                    $this->request['route'][$this->route_setting['set'][$k]] =  $v;
                }
            }
        }
        $this->response = $class->$me($this->request,$this->request['route']);
    }
    private function show()
    {
        $app = Config::get("app");
        foreach($app as $conf=>$val){
            $this->agrs[$conf] = $val;
        }

        $this->agrs['UserInfo'] = Session::get('UserInfo');

        foreach($this->response as $key=>$value){
            $this->agrs[$key] = $value;
        }
        $this->agrs['route']= $this->route;
        $this->agrs['route_setting'] = $this->route_setting;

        //URLパラメータ優先
        if(Helper::ak("route",$this->request) && count($this->request['route']) > 0){
            foreach($this->request['route'] as $uname=>$uvalue){
                $this->agrs[$uname] = $uvalue;
            }
        }
        $loader = new \Twig\Loader\FilesystemLoader(Dir::views_path());
        $twig = new \Twig\Environment($loader, [
            'debug' => Config::get('app','debug'),
        ]);
        
        $this->addFunc($twig);
        
        if($this->route_setting['view'] === 'ajax'){
            $this->agrs['response'] = $this->response;
            echo $twig->render('template/ajax.html', $this->agrs);
            return;
        }
        header("X-FRAME-OPTIONS:DENY");
        header("Cache-Control:no-cache,must-revalidate");
        header("Expires:".date("Y-m-d H:i:s",time() - (60*61)));
        header("Content-type: text/html;charset=UTF-8");
        
        echo $twig->render('template/frame/main.html', $this->agrs);
    }

    private function urlana()
    {
        $url = $_SERVER['REQUEST_URI'];
        $urlAr = explode("/",$url);
        foreach($urlAr as $d){
            if($d != str_replace("/","",Config::get("app","root")) and $d != '' and $d != null){
                $this->urls[] = strtok($d, '?'); //QueryStringは抜く
            }
        }
    }
    
    private function addFunc(&$twig){
        
        $datef = new \Twig_SimpleFunction('date_format', function ($format,$str=null) {
            $str = ($str) ?? time();
            return date($format,$str);
        });
        $twig->addFunction($datef);
        
        $titler = new \Twig_SimpleFunction('titler', function ($title=null) {
            return !is_null($title) ? '<span class="card-title" id="page_title">'.Html::hsp($title).'</span>':'';
        });
        $twig->addFunction($titler);
    }
}