<?php
namespace komaphp\db;

use komaphp\core\Config as Config;
use komaphp\core\Log as Log;
use komaphp\core\Helper as Helper;

class Exec {
    private $c = null;
    private $stmt = null;
    function __construct()
    {
        $dbselect = Config::get('database','db','mysql');
        $connect_info = Config::get('database',$dbselect);
        try{
            if($dbselect == 'sqlite'){
                $this->c = new \PDO('sqlite:'.$connect_info['dbname'].'.db',[
                    \PDO::ATTR_ERRMODE => \PDO::ERRMODE_EXCEPTION,
                    \PDO::ATTR_DEFAULT_FETCH_MODE => \PDO::FETCH_ASSOC,
                ]);
            }else{
                $this->c = new \PDO($dbselect.':host='.$connect_info['host'].';dbname='.$connect_info['dbname'],$connect_info['user'],$connect_info['pass'],[
                    \PDO::ATTR_ERRMODE => \PDO::ERRMODE_EXCEPTION,
                    \PDO::ATTR_DEFAULT_FETCH_MODE => \PDO::FETCH_ASSOC,
                    \PDO::MYSQL_ATTR_INIT_COMMAND => 'SET NAMES '.Config::get('database','charset','utf8')
                ]);
            }
        }catch (\PDOException $e){
            $msg = Helper::jsonen($e);
            Log::set($msg);
            die($msg);
        }
    }

    public function executeSql(string $sql=null,array $param=[]):\PDOStatement
    {
        if(is_null($sql) || empty($sql)) return [];
        try{
            $this->c->beginTransaction();
            $this->stmt = $this->c->prepare($sql);
            Log::set($sql);
            if(count($param) > 0){
                $this->bindValueData($param);
            }
            if(!$this->stmt->execute()){
                throw new \PDOException($this->stmt->errorInfo());
            }
            $this->c->commit();
            return $this->stmt;
        }catch (\PDOException $e){
            $this->c->rollBack();
            $msg = Helper::jsonen($e);
            Log::set($msg);
            die($msg);
        }
    }

    public function create(string $table,array $param=[]):\PDOStatement
    {
        $clms = [];
        $vals = [];
        foreach($param as $clm=>$set){
            $clms[] = $clm;
            $vals[] = ':'.$clm;
        }
        if(count($param) === 0){
            throw new \PDOException(['errorInfo'=>'作成先カラムが1つも指定されていません']);
        }

        $sql = 'INSERT INTO '.$table.' '.implode(',',$clms).' VALUES ('.implode(',',$vals).')';
        return $this->executeSql($sql,$param);
    }
    public function update(string $table,array $param=[],array $whereParam = []):\PDOStatement
    {
        $clms = [];
        foreach($param as $clm=>$set){
            $clms[] = $clm.'='.':'.$clm;
        }
        if(count($param) === 0){
            throw new \PDOException(['errorInfo'=>'更新先カラムが1つも指定されていません']);
        }

        $sql = 'UPDATE '.$table.' SET '.implode(',',$clms);
        if(count($whereParam) > 0){
            $sql .= $this->whereCreate($whereParam);
        }

        return $this->executeSql($sql,$param);
    }

    public function read(string $table,array $select_clm = [],array $param=[],array $order = [],bool $count = false,array $offset_limit = []):array
    {
        $sc = (count($select_clm) > 0)? implode(',',$select_clm): '*';

        if($count){
            $sc = ' COUNT(*) ';
        }

        $sql = 'SELECT '.$sc.' FROM '.$table;
        if(count($param) > 0){
            $sql .= $this->whereCreate($param);
        }
        if(count($order) > 0){
            $sql .= $this->orderCreate($order);
        }
        if(!$count && count($offset_limit) > 0){
            $param['offset'] = ['param'=>\PDO::PARAM_INT,'value'=>$offset_limit[0]];
            $param['limit'] = ['param'=>\PDO::PARAM_INT,'value'=>$offset_limit[1]];
            switch(Config::get('database','db') ){
                case 'mysql':
                    $sql .= ' LIMIT :limit,:offset';
                    break;
                case 'pgsql':
                    $sql .= ' LIMIT :limit OFFSET :offset';
                    break;
                    //他は未対応

            }
        }

        return $this->executeSql($sql,$param,$offset_limit)->fetchAll(\PDO::FETCH_ASSOC);
    }

    private function whereCreate(array $param = []):string
    {
        $wheresql = '';
        if(count($param) > 0){
            $whereclms = [];
            foreach($param as $clm=>$set){
                $s = '=';
                $v = ':'.$clm;
                if(Helper::ak('type',$set)){
                    $s = $set['type'];
                }
                if(($s==='in' || $s==='IN') && count($set['value']) > 0){
                    $dda = [];
                    foreach($set['value'] as $ini=>$vd){
                        $dda[] = ':'.$clm.$ini;
                    }
                    $whereclms[] = $clm.' IN ('.implode(',',$dda).')';
                }elseif(($s==='term' || $s==='TERM') && count($set['value']) > 0){
                        $whereclms[] = $clm.' between :'.$clm.'0 AND :'.$clm.'1';
                }else{
                    $whereclms[] = $clm.$s.$v;
                }

            }

            $wheresql = ' WHERE '.implode(' AND ',$whereclms);//and以外はあとで考える
        }
        return $wheresql;
    }

    //0 asc 1 desc
    private function orderCreate(array $order = []):string
   {
       $orderk = [];
       foreach($order as $clm=>$od){
           switch($od)
           {
               case 0:
                   $orderk[] = $clm.' ASC';
                   break;
               case 1:
                   $orderk[] = $clm.' DESC';
                   break;
           }
       }

       return ' ORDER BY '.implode(',',$orderk);
    }
    private function bindValueData($param=[])
    {
        foreach($param as $clm=>$set){
            $prm_set = \PDO::PARAM_STR;
            $prm_val = $set['value'];

            if(Helper::ak('type',$set)){
                switch($set['type']){
                    case 'like':
                        $prm_val = '%' . preg_replace('/(?=[!_%])/', '!', $prm_val) . '%';
                        break;
                }
                if(($set['type']==='in' || $set['type']==='IN') && count($prm_val) > 0){
                    foreach($prm_val as $ini=>$vd){
                        $this->stmt->bindValue(':'.$clm.$ini,$vd,$prm_set);
                    }
                }elseif(($set['type']==='term' || $set['type']==='TERM') && count($prm_val) > 0){
                    $this->stmt->bindValue(':'.$clm.'0',$prm_val[0],$prm_set);
                    $this->stmt->bindValue(':'.$clm.'1',$prm_val[1],$prm_set);

                }else{
                    if(Helper::ak('param',$set)){
                        $prm_set = $set['param'];
                    }
                    $this->stmt->bindValue(':'.$clm,$prm_val,$prm_set);
                }
            }else{
                if(Helper::ak('param',$set)){
                    $prm_set = $set['param'];
                }
                $this->stmt->bindValue(':'.$clm,$prm_val,$prm_set);
            }
        }
    }
}