<?php
namespace komaphp\db;

use komaphp\core\Config as Config;
use komaphp\core\Log as Log;
use komaphp\core\Helper as Helper;
use komaphp\core\Input as Input;

class Model {
    protected $table;
    protected $properties = [];
    protected $order = ['update_date'=>0];
    protected $columns = [];
    public $prop = [];
    
    function __set($name,$value =null){
        $this->prop[$name] = $value;
    }
    function __get($name){
        return $this->prop[$name];
    }
    
    protected function setTableInfo(string $model_name ):bool
    {
        if(!$model_name || empty($model_name)) return false;

        $ex = explode('\\',$model_name);
        $set = Config::get('db_tables',end($ex));
        $this->table = Config::get('database','prefix').'_'.$set['table_name'];
        $this->columns = $set['columns'];
        return true;
    }

    function insert($inputName = null,$addD=[]){
        return $this->runExec(__METHOD__,$inputName,$addD,[]);
    }
    function update($inputName = null,$addD=[],$where=[]){
        return $this->runExec(__METHOD__,$inputName,$addD,$where);
    }
    function delete($inputName = null,$where=[]){
        return $this->runExec(__METHOD__,$inputName,[],$where);
    }
    private function runExec(string $type,string $input_name,array $param=[],array $where=[])
    {
        $type = str_replace(__CLASS__,'',$type);
        $type = str_replace('::','',$type);
        $inputconfig = Config::get('input',$input_name);

        $upDatingParam = [];

        foreach($this->columns as $clm=>$set){
            if(Helper::ak($clm,$inputconfig)){
                $upDatingParam[$clm] = Input::get($inputconfig[$clm]['name']);
            }
            if(Helper::ak($clm,$param)){
                $upDatingParam[$clm] = $param[$clm];
            }
        }

        $now = date('Y-m-d H:i:s');
        $upDatingParam['update_date'] = $now;

        $exec_function = '';
        switch($type){
            case 'insert':
                $upDatingParam['create_date'] = $now;
                $exec_function = 'create';
                break;
            case 'update':
                $exec_function = 'update';
                break;
        }
        $exec = new Exec();

        return $exec->$exec_function($this->table,$this->paramToDparam($upDatingParam),$this->paramToDparam($where));
    }
    function query($sql,array $param=[]):\PDOStatement
    {
        $exec = new Exec();
        Log::set('SQL:'.$sql);
        return $exec->executeSql($sql,$this->paramToDparam($param));
    }

    //Read
    function select(array $clm=[],array $param=[],array $order = [],bool $count = false,array $offset_limit = []):array
    {
        $exec = new Exec();
        return $exec->read($this->table,$clm,$this->paramToDparam($param),$order,$count,$offset_limit);
    }
    function count(array $param=[]){
        $list = $this->select([],$param,[],true);
        return $list[0]['count'];

    }
    function select_pager_data(array $param=[],array $order = [],array $request = []):array
    {
        if(count($request) === 0){
            $offset = 0;
            $limit = Config::get('app','pager');
        }else{
            if(Helper::ak('route',$request) && Helper::ak('page_number',$request['route'])) {
                $pager_number = $request['route']['page_number'];
                $limit = (empty($request['router']['pager']) || is_null($request['router']['pager']) || !is_numeric($request['router']['pager'])) ? Config::get('app','pager') : $request['router']['pager'];
                $offset = ($pager_number === 1 || empty($pager_number) || is_null($pager_number) || !is_numeric($pager_number)) ? 0 : (($pager_number-1) * $limit) ;
            }else{
                $offset = 0;
                $limit = Config::get('app','pager');
            }
        }
        return $this->select([],$param,$order,false,[$offset,$limit]);
    }
    function getOneInfo(array $clm=[],array $param=[]):array
    {
        $record = $this->select($clm,$param);
        if(count($record)>0){
            return $record[0];
        }else{
            return [];
        }
    }
    function getByKey(array $keysvalue = []):array
    {
        $prm = [];
        $i = 0;
        foreach($this->columns as $clm=>$set){//第1キーから順番に渡される前提
            if(Helper::ak('pk',$set)){
                $prm[$clm] = $keysvalue[$i];
                $i++;
            }
        }
        return $this->getOneInfo([],$prm);

    }

    private function paramToDparam(array $param=[]):array
    {
        $dparam = [];
        foreach($param as $clmname=>$value){
            $p = [];
            if(Helper::ak($clmname,$this->columns)){
                $p['param'] = $this->columns[$clmname]['type'];
                if(is_array($value) && Helper::ak('type',$value)){
                    $p['type'] = $value['type'];
                    $p['value'] = $value['value'];
                }else{
                    $p['type'] = '=';
                    $p['value'] = $value;
                }
                $dparam[$clmname] = $p;
            }
        }
        return $dparam;
    }
    public function creatRanDomId(){
        $mid = rand(1,time());
        $d = $this->getByKey([$mid]);
        if(count($d) === 0){
            return $mid;
        }else{
            $this->creatRanDomId();
        }
    }
}